import pytest

from dirty_equals import (
    IsApprox,
    IsFloat,
    IsFloatInf,
    IsFloatInfNeg,
    IsFloatInfPos,
    IsFloatNan,
    IsInt,
    IsNegative,
    IsNegativeFloat,
    IsNegativeInt,
    IsNonNegative,
    IsNonPositive,
    IsPositive,
    IsPositiveFloat,
    IsPositiveInt,
)


@pytest.mark.parametrize(
    'other,dirty',
    [
        (1, IsInt),
        (1, IsInt()),
        (1, IsInt(exactly=1)),
        (1, IsPositiveInt),
        (-1, IsNegativeInt),
        (-1.0, IsFloat),
        (-1.0, IsFloat(exactly=-1.0)),
        (1.0, IsPositiveFloat),
        (-1.0, IsNegativeFloat),
        (1, IsPositive),
        (1.0, IsPositive),
        (-1, IsNegative),
        (-1.0, IsNegative),
        (5, IsInt(gt=4)),
        (5, IsInt(ge=5)),
        (5, IsInt(lt=6)),
        (5, IsInt(le=5)),
        (1, IsApprox(1)),
        (1, IsApprox(2, delta=1)),
        (100, IsApprox(99)),
        (-100, IsApprox(-99)),
        (0, IsNonNegative),
        (1, IsNonNegative),
        (0.0, IsNonNegative),
        (1.0, IsNonNegative),
        (0, IsNonPositive),
        (-1, IsNonPositive),
        (0.0, IsNonPositive),
        (-1.0, IsNonPositive),
        (-1, IsNonPositive & IsInt),
        (1, IsNonNegative & IsInt),
        (float('inf'), IsFloatInf),
        (-float('inf'), IsFloatInf),
        (float('-inf'), IsFloatInf),
        (float('inf'), IsFloatInfPos),
        (-float('-inf'), IsFloatInfPos),
        (-float('inf'), IsFloatInfNeg),
        (float('-inf'), IsFloatInfNeg),
        (float('nan'), IsFloatNan),
        (-float('nan'), IsFloatNan),
        (float('-nan'), IsFloatNan),
    ],
)
def test_dirty_equals(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        (1.0, IsInt),
        (1.2, IsInt),
        (1, IsInt(exactly=2)),
        (True, IsInt),
        (False, IsInt),
        (1.0, IsInt()),
        (-1, IsPositiveInt),
        (0, IsPositiveInt),
        (1, IsNegativeInt),
        (0, IsNegativeInt),
        (1, IsFloat),
        (1, IsFloat(exactly=1.0)),
        (1.1234, IsFloat(exactly=1.0)),
        (-1.0, IsPositiveFloat),
        (0.0, IsPositiveFloat),
        (1.0, IsNegativeFloat),
        (0.0, IsNegativeFloat),
        (-1, IsPositive),
        (-1.0, IsPositive),
        (4, IsInt(gt=4)),
        (4, IsInt(ge=5)),
        (6, IsInt(lt=6)),
        (6, IsInt(le=5)),
        (-1, IsNonNegative),
        (-1.0, IsNonNegative),
        (1, IsNonPositive),
        (1.0, IsNonPositive),
        (-1.0, IsNonPositive & IsInt),
        (1.0, IsNonNegative & IsInt),
        (1, IsFloatNan),
        (1.0, IsFloatNan),
        (1, IsFloatInf),
        (1.0, IsFloatInf),
        (-float('inf'), IsFloatInfPos),
        (float('-inf'), IsFloatInfPos),
        (-float('-inf'), IsFloatInfNeg),
        (-float('-inf'), IsFloatInfNeg),
    ],
    ids=repr,
)
def test_dirty_not_equals(other, dirty):
    assert other != dirty


def test_invalid_approx_gt():
    with pytest.raises(TypeError, match='"approx" cannot be combined with "gt", "lt", "ge", or "le"'):
        IsInt(approx=1, gt=1)


def test_invalid_exactly_approx():
    with pytest.raises(TypeError, match='"exactly" cannot be combined with "approx"'):
        IsInt(exactly=1, approx=1)


def test_invalid_exactly_gt():
    with pytest.raises(TypeError, match='"exactly" cannot be combined with "gt", "lt", "ge", or "le"'):
        IsInt(exactly=1, gt=1)


def test_not_int():
    d = IsInt()
    with pytest.raises(AssertionError):
        assert '1' == d
    assert repr(d) == 'IsInt()'


def test_not_negative():
    d = IsNegativeInt
    with pytest.raises(AssertionError):
        assert 1 == d
    assert repr(d) == 'IsNegativeInt'



import logging
import os
import re

from mkdocs.config import Config
from mkdocs.structure.files import Files
from mkdocs.structure.pages import Page

try:
    import pytest
except ImportError:
    pytest = None

logger = logging.getLogger('mkdocs.test_examples')


def on_pre_build(config: Config):
    pass


def on_files(files: Files, config: Config) -> Files:
    return remove_files(files)


def remove_files(files: Files) -> Files:
    to_remove = []
    for file in files:
        if file.src_path in {'plugins.py'}:
            to_remove.append(file)
        elif file.src_path.startswith('__pycache__/'):
            to_remove.append(file)

    logger.debug('removing files: %s', [f.src_path for f in to_remove])
    for f in to_remove:
        files.remove(f)

    return files


def on_page_markdown(markdown: str, page: Page, config: Config, files: Files) -> str:
    markdown = remove_code_fence_attributes(markdown)
    return add_version(markdown, page)


def add_version(markdown: str, page: Page) -> str:
    if page.file.src_uri == 'index.md':
        version_ref = os.getenv('GITHUB_REF')
        if version_ref and version_ref.startswith('refs/tags/'):
            version = re.sub('^refs/tags/', '', version_ref.lower())
            url = f'https://github.com/samuelcolvin/dirty-equals/releases/tag/{version}'
            version_str = f'Documentation for version: [{version}]({url})'
        elif sha := os.getenv('GITHUB_SHA'):
            sha = sha[:7]
            url = f'https://github.com/samuelcolvin/dirty-equals/commit/{sha}'
            version_str = f'Documentation for development version: [{sha}]({url})'
        else:
            version_str = 'Documentation for development version'
        markdown = re.sub(r'{{ *version *}}', version_str, markdown)
    return markdown


def remove_code_fence_attributes(markdown: str) -> str:
    """
    There's no way to add attributes to code fences that works with both pycharm and mkdocs, hence we use
    `py key="value"` to provide attributes to pytest-examples, then remove those attributes here.

    https://youtrack.jetbrains.com/issue/IDEA-297873 & https://python-markdown.github.io/extensions/fenced_code_blocks/
    """

    def remove_attrs(match: re.Match[str]) -> str:
        suffix = re.sub(
            r' (?:test|lint|upgrade|group|requires|output|rewrite_assert)=".+?"', '', match.group(2), flags=re.M
        )
        return f'{match.group(1)}{suffix}'

    return re.sub(r'^( *``` *py)(.*)', remove_attrs, markdown, flags=re.M)



import sys
import uuid
from dataclasses import dataclass
from enum import Enum, auto
from hashlib import md5, sha1, sha256
from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network

import pytest

from dirty_equals import (
    FunctionCheck,
    IsDataclass,
    IsDataclassType,
    IsEnum,
    IsHash,
    IsInt,
    IsIP,
    IsJson,
    IsPartialDataclass,
    IsStr,
    IsStrictDataclass,
    IsUrl,
    IsUUID,
)


class FooEnum(Enum):
    a = auto()
    b = auto()
    c = 'c'


@dataclass
class Foo:
    a: int
    b: int
    c: str


foo = Foo(1, 2, 'c')


@dataclass
class Address:
    street: str
    zip_code: str


@dataclass
class Person:
    name: str
    address: Address


person = Person(name='Alice', address=Address(street='123 Main St', zip_code='12345'))


@pytest.mark.parametrize(
    'other,dirty',
    [
        (uuid.uuid4(), IsUUID()),
        (uuid.uuid4(), IsUUID),
        (uuid.uuid4(), IsUUID(4)),
        ('edf9f29e-45c7-431c-99db-28ea44df9785', IsUUID),
        ('edf9f29e-45c7-431c-99db-28ea44df9785', IsUUID(4)),
        ('edf9f29e45c7431c99db28ea44df9785', IsUUID(4)),
        (uuid.uuid3(uuid.UUID('edf9f29e-45c7-431c-99db-28ea44df9785'), 'abc'), IsUUID),
        (uuid.uuid3(uuid.UUID('edf9f29e-45c7-431c-99db-28ea44df9785'), 'abc'), IsUUID(3)),
        (uuid.uuid1(), IsUUID(1)),
        (str(uuid.uuid1()), IsUUID(1)),
        ('ea9e828d-fd18-3898-99f3-5a46dbcee036', IsUUID(3)),
    ],
)
def test_is_uuid_true(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        ('foobar', IsUUID()),
        ([1, 2, 3], IsUUID()),
        ('edf9f29e-45c7-431c-99db-28ea44df9785', IsUUID(5)),
        (uuid.uuid3(uuid.UUID('edf9f29e-45c7-431c-99db-28ea44df9785'), 'abc'), IsUUID(4)),
        (uuid.uuid1(), IsUUID(4)),
        ('edf9f29e-45c7-431c-99db-28ea44df9785', IsUUID(1)),
        ('ea9e828d-fd18-3898-99f3-5a46dbcee036', IsUUID(4)),
    ],
)
def test_is_uuid_false(other, dirty):
    assert other != dirty


def test_is_uuid_false_repr():
    is_uuid = IsUUID()
    with pytest.raises(AssertionError):
        assert '123' == is_uuid
    assert str(is_uuid) == 'IsUUID()'


def test_is_uuid4_false_repr():
    is_uuid = IsUUID(4)
    with pytest.raises(AssertionError):
        assert '123' == is_uuid
    assert str(is_uuid) == 'IsUUID(4)'


@pytest.mark.parametrize('json_value', ['null', '"xyz"', '[1, 2, 3]', '{"a": 1}'])
def test_is_json_any_true(json_value):
    assert json_value == IsJson()
    assert json_value == IsJson


def test_is_json_any_false():
    is_json = IsJson()
    with pytest.raises(AssertionError):
        assert 'foobar' == is_json
    assert str(is_json) == 'IsJson()'


@pytest.mark.parametrize(
    'json_value,expected_value',
    [
        ('null', None),
        ('"xyz"', 'xyz'),
        ('[1, 2, 3]', [1, 2, 3]),
        ('{"a": 1}', {'a': 1}),
    ],
)
def test_is_json_specific_true(json_value, expected_value):
    assert json_value == IsJson(expected_value)
    assert json_value == IsJson[expected_value]


def test_is_json_invalid():
    assert 'invalid json' != IsJson
    assert 123 != IsJson
    assert [1, 2] != IsJson


def test_is_json_kwargs():
    assert '{"a": 1, "b": 2}' == IsJson(a=1, b=2)
    assert '{"a": 1, "b": 3}' != IsJson(a=1, b=2)


def test_is_json_specific_false():
    is_json = IsJson([1, 2, 3])
    with pytest.raises(AssertionError):
        assert '{"a": 1}' == is_json
    assert str(is_json) == 'IsJson([1, 2, 3])'


def test_equals_function():
    func_argument = None

    def foo(v):
        nonlocal func_argument
        func_argument = v
        return v % 2 == 0

    assert 4 == FunctionCheck(foo)
    assert func_argument == 4
    assert 5 != FunctionCheck(foo)


def test_equals_function_fail():
    def foobar(v):
        return False

    c = FunctionCheck(foobar)

    with pytest.raises(AssertionError):
        assert 4 == c

    assert str(c) == 'FunctionCheck(foobar)'


def test_json_both():
    with pytest.raises(TypeError, match='IsJson requires either an argument or kwargs, not both'):
        IsJson(1, a=2)


@pytest.mark.parametrize(
    'other,dirty',
    [
        (IPv4Address('127.0.0.1'), IsIP()),
        pytest.param(
            IPv4Network('43.48.0.0/12'),
            IsIP(),
            marks=pytest.mark.xfail(
                sys.version_info >= (3, 14), reason='https://github.com/python/cpython/issues/141647'
            ),
        ),
        (IPv6Address('::eeff:ae3f:d473'), IsIP()),
        pytest.param(
            IPv6Network('::eeff:ae3f:d473/128'),
            IsIP(),
            marks=pytest.mark.xfail(
                sys.version_info >= (3, 14), reason='https://github.com/python/cpython/issues/141647'
            ),
        ),
        ('2001:0db8:0a0b:12f0:0000:0000:0000:0001', IsIP()),
        ('179.27.154.96', IsIP),
        ('43.62.123.119', IsIP(version=4)),
        ('::ffff:2b3e:7b77', IsIP(version=6)),
        ('0:0:0:0:0:ffff:2b3e:7b77', IsIP(version=6)),
        ('54.43.53.219/10', IsIP(version=4, netmask='255.192.0.0')),
        ('::ffff:aebf:d473/12', IsIP(version=6, netmask='fff0::')),
        ('2001:0db8:0a0b:12f0:0000:0000:0000:0001', IsIP(version=6)),
        (3232235521, IsIP()),
        (b'\xc0\xa8\x00\x01', IsIP()),
        (338288524927261089654018896845572831328, IsIP(version=6)),
        (b'\x20\x01\x06\x58\x02\x2a\xca\xfe\x02\x00\x00\x00\x00\x00\x00\x01', IsIP(version=6)),
    ],
    ids=repr,
)
def test_is_ip_true(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        ('foobar', IsIP()),
        ([1, 2, 3], IsIP()),
        ('210.115.28.193', IsIP(version=6)),
        ('::ffff:d273:1cc1', IsIP(version=4)),
        ('210.115.28.193/12', IsIP(version=6, netmask='255.255.255.0')),
        ('::ffff:d273:1cc1', IsIP(version=6, netmask='fff0::')),
        (3232235521, IsIP(version=6)),
        (338288524927261089654018896845572831328, IsIP(version=4)),
    ],
)
def test_is_ip_false(other, dirty):
    assert other != dirty


def test_not_ip_repr():
    is_ip = IsIP()
    with pytest.raises(AssertionError):
        assert '123' == is_ip
    assert str(is_ip) == 'IsIP()'


def test_ip_bad_netmask():
    with pytest.raises(TypeError, match='To check the netmask you must specify the IP version'):
        IsIP(netmask='255.255.255.0')


@pytest.mark.parametrize(
    'other,dirty',
    [
        ('f1e069787ECE74531d112559945c6871', IsHash('md5')),
        ('40bd001563085fc35165329ea1FF5c5ecbdbbeef', IsHash('sha-1')),
        ('a665a45920422f9d417e4867eFDC4fb8a04a1f3fff1fa07e998e86f7f7a27ae3', IsHash('sha-256')),
        (b'f1e069787ECE74531d112559945c6871', IsHash('md5')),
        (bytearray(b'f1e069787ECE74531d112559945c6871'), IsHash('md5')),
    ],
)
def test_is_hash_true(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        ('foobar', IsHash('md5')),
        (b'\x81 UnicodeDecodeError', IsHash('md5')),
        ([1, 2, 3], IsHash('sha-1')),
        ('f1e069787ECE74531d112559945c6871d', IsHash('md5')),
        ('400bd001563085fc35165329ea1FF5c5ecbdbbeef', IsHash('sha-1')),
        ('a665a45920422g9d417e4867eFDC4fb8a04a1f3fff1fa07e998e86f7f7a27ae3', IsHash('sha-256')),
    ],
)
def test_is_hash_false(other, dirty):
    assert other != dirty


@pytest.mark.parametrize(
    'hash_type',
    ['md5', 'sha-1', 'sha-256'],
)
def test_is_hash_md5_false_repr(hash_type):
    is_hash = IsHash(hash_type)
    with pytest.raises(AssertionError):
        assert '123' == is_hash
    assert str(is_hash) == f"IsHash('{hash_type}')"


@pytest.mark.parametrize(
    'hash_func, hash_type',
    [(md5, 'md5'), (sha1, 'sha-1'), (sha256, 'sha-256')],
)
def test_hashlib_hashes(hash_func, hash_type):
    assert hash_func(b'dirty equals').hexdigest() == IsHash(hash_type)


def test_wrong_hash_type():
    with pytest.raises(ValueError, match='Hash type must be one of the following values: md5, sha-1, sha-256'):
        assert '123' == IsHash('ntlm')


@pytest.mark.parametrize(
    'other,dirty',
    [
        ('https://example.com', IsUrl),
        ('https://example.com', IsUrl(scheme='https')),
        ('postgres://user:pass@localhost:5432/app', IsUrl(postgres_dsn=True)),
    ],
)
def test_is_url_true(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        ('https://example.com', IsUrl(postgres_dsn=True)),
        ('https://example.com', IsUrl(scheme='http')),
        ('definitely not a url', IsUrl),
        (42, IsUrl),
        ('https://anotherexample.com', IsUrl(postgres_dsn=True)),
    ],
)
def test_is_url_false(other, dirty):
    assert other != dirty


def test_is_url_invalid_kwargs():
    with pytest.raises(
        TypeError,
        match='IsURL only checks these attributes: scheme, host, host_type, user, password, port, path, query, '
        'fragment',
    ):
        IsUrl(https=True)


def test_is_url_too_many_url_types():
    with pytest.raises(
        ValueError,
        match='You can only check against one Pydantic url type at a time',
    ):
        assert 'https://example.com' == IsUrl(any_url=True, http_url=True, postgres_dsn=True)


@pytest.mark.parametrize(
    'other,dirty',
    [
        (Foo, IsDataclassType),
        (Foo, IsDataclassType()),
    ],
)
def test_is_dataclass_type_true(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        (foo, IsDataclassType),
        (foo, IsDataclassType()),
        (Foo, IsDataclass),
    ],
)
def test_is_dataclass_type_false(other, dirty):
    assert other != dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        (foo, IsDataclass),
        (foo, IsDataclass()),
        (foo, IsDataclass(a=IsInt, b=2, c=IsStr)),
        (foo, IsDataclass(a=1, c='c', b=2).settings(strict=False, partial=False)),
        (foo, IsDataclass(a=1, b=2, c='c').settings(strict=True, partial=False)),
        (foo, IsStrictDataclass(a=1, b=2, c='c')),
        (foo, IsDataclass(c='c', a=1).settings(strict=False, partial=True)),
        (foo, IsPartialDataclass(c='c', a=1)),
        (foo, IsDataclass(b=2, c='c').settings(strict=True, partial=True)),
        (foo, IsStrictDataclass(b=2, c='c').settings(partial=True)),
        (foo, IsPartialDataclass(b=2, c='c').settings(strict=True)),
    ],
)
def test_is_dataclass_true(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        (foo, IsDataclassType),
        (Foo, IsDataclass),
        (foo, IsDataclass(a=1)),
        (foo, IsDataclass(a=IsStr, b=IsInt, c=IsStr)),
        (foo, IsDataclass(b=2, a=1, c='c').settings(strict=True)),
        (foo, IsStrictDataclass(b=2, a=1, c='c')),
        (foo, IsDataclass(b=2, a=1).settings(partial=False)),
    ],
)
def test_is_dataclass_false(other, dirty):
    assert other != dirty


def test_is_dataclass_nested():
    assert person == IsDataclass(name='Alice', address=IsDataclass(street='123 Main St', zip_code='12345'))


@pytest.mark.parametrize(
    'other,dirty',
    [
        (FooEnum.a, IsEnum),
        (FooEnum.b, IsEnum(FooEnum)),
        (2, IsEnum(FooEnum)),
        ('c', IsEnum(FooEnum)),
    ],
)
def test_is_enum_true(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        (FooEnum, IsEnum),
        (FooEnum, IsEnum(FooEnum)),
        (4, IsEnum(FooEnum)),
    ],
)
def test_is_enum_false(other, dirty):
    assert other != dirty



from datetime import date, datetime, timedelta, timezone
from unittest.mock import Mock
from zoneinfo import ZoneInfo

import pytest

from dirty_equals import IsDate, IsDatetime, IsNow, IsToday


@pytest.mark.parametrize(
    'value,dirty,expect_match',
    [
        pytest.param(datetime(2000, 1, 1), IsDatetime(approx=datetime(2000, 1, 1)), True, id='same'),
        # Note: this requires the system timezone to be UTC
        pytest.param(946684800, IsDatetime(approx=datetime(2000, 1, 1), unix_number=True), True, id='unix-int'),
        # Note: this requires the system timezone to be UTC
        pytest.param(946684800.123, IsDatetime(approx=datetime(2000, 1, 1), unix_number=True), True, id='unix-float'),
        pytest.param(946684800, IsDatetime(approx=datetime(2000, 1, 1)), False, id='unix-different'),
        pytest.param(
            '2000-01-01T00:00', IsDatetime(approx=datetime(2000, 1, 1), iso_string=True), True, id='iso-string-true'
        ),
        pytest.param('2000-01-01T00:00', IsDatetime(approx=datetime(2000, 1, 1)), False, id='iso-string-different'),
        pytest.param('broken', IsDatetime(approx=datetime(2000, 1, 1)), False, id='iso-string-wrong'),
        pytest.param(
            '28/01/87', IsDatetime(approx=datetime(1987, 1, 28), format_string='%d/%m/%y'), True, id='string-format'
        ),
        pytest.param('28/01/87', IsDatetime(approx=datetime(2000, 1, 1)), False, id='string-format-different'),
        pytest.param('foobar', IsDatetime(approx=datetime(2000, 1, 1)), False, id='string-format-wrong'),
        pytest.param(datetime(2000, 1, 1).isoformat(), IsNow(iso_string=True), False, id='isnow-str-different'),
        pytest.param([1, 2, 3], IsDatetime(approx=datetime(2000, 1, 1)), False, id='wrong-type'),
        pytest.param(
            datetime(2020, 1, 1, 12, 13, 14), IsDatetime(approx=datetime(2020, 1, 1, 12, 13, 14)), True, id='tz-same'
        ),
        pytest.param(
            datetime(2020, 1, 1, 12, 13, 14, tzinfo=timezone.utc),
            IsDatetime(approx=datetime(2020, 1, 1, 12, 13, 14), enforce_tz=False),
            True,
            id='tz-utc',
        ),
        pytest.param(
            datetime(2020, 1, 1, 12, 13, 14, tzinfo=timezone.utc),
            IsDatetime(approx=datetime(2020, 1, 1, 12, 13, 14)),
            False,
            id='tz-utc-different',
        ),
        pytest.param(
            datetime(2020, 1, 1, 12, 13, 14),
            IsDatetime(approx=datetime(2020, 1, 1, 12, 13, 14, tzinfo=timezone.utc), enforce_tz=False),
            False,
            id='tz-approx-tz',
        ),
        pytest.param(
            datetime(2020, 1, 1, 12, 13, 14, tzinfo=timezone(offset=timedelta(hours=1))),
            IsDatetime(approx=datetime(2020, 1, 1, 12, 13, 14), enforce_tz=False),
            True,
            id='tz-1-hour',
        ),
        pytest.param(
            datetime(2022, 2, 15, 15, 15, tzinfo=ZoneInfo('Europe/London')),
            IsDatetime(approx=datetime(2022, 2, 15, 10, 15, tzinfo=ZoneInfo('America/New_York')), enforce_tz=False),
            True,
            id='tz-both-tz',
        ),
        pytest.param(
            datetime(2022, 2, 15, 15, 15, tzinfo=ZoneInfo('Europe/London')),
            IsDatetime(approx=datetime(2022, 2, 15, 10, 15, tzinfo=ZoneInfo('America/New_York'))),
            False,
            id='tz-both-tz-different',
        ),
        pytest.param(datetime(2000, 1, 1), IsDatetime(ge=datetime(2000, 1, 1)), True, id='ge'),
        pytest.param(datetime(1999, 1, 1), IsDatetime(ge=datetime(2000, 1, 1)), False, id='ge-not'),
        pytest.param(datetime(2000, 1, 2), IsDatetime(gt=datetime(2000, 1, 1)), True, id='gt'),
        pytest.param(datetime(2000, 1, 1), IsDatetime(gt=datetime(2000, 1, 1)), False, id='gt-not'),
    ],
)
def test_is_datetime(value, dirty, expect_match):
    if expect_match:
        assert value == dirty
    else:
        assert value != dirty


@pytest.mark.skipif(ZoneInfo is None, reason='requires zoneinfo')
def test_is_datetime_zoneinfo():
    london = datetime(2022, 2, 15, 15, 15, tzinfo=ZoneInfo('Europe/London'))
    ny = datetime(2022, 2, 15, 10, 15, tzinfo=ZoneInfo('America/New_York'))
    assert london != IsDatetime(approx=ny)
    assert london == IsDatetime(approx=ny, enforce_tz=False)


def test_is_now_dt():
    is_now = IsNow()
    dt = datetime.now()
    assert dt == is_now
    assert str(is_now) == repr(dt)


def test_is_now_str():
    assert datetime.now().isoformat() == IsNow(iso_string=True)


def test_repr():
    v = IsDatetime(approx=datetime(2032, 1, 2, 3, 4, 5), iso_string=True)
    assert str(v) == 'IsDatetime(approx=datetime.datetime(2032, 1, 2, 3, 4, 5), iso_string=True)'


@pytest.mark.skipif(ZoneInfo is None, reason='requires zoneinfo')
def test_is_now_tz():
    utc_now = datetime.now(timezone.utc)
    now_ny = utc_now.astimezone(ZoneInfo('America/New_York'))
    assert now_ny == IsNow(tz='America/New_York')
    # depends on the time of year and DST
    assert now_ny == IsNow(tz=timezone(timedelta(hours=-5))) | IsNow(tz=timezone(timedelta(hours=-4)))

    now = datetime.now()
    assert now == IsNow
    assert now.timestamp() == IsNow(unix_number=True)
    assert now.timestamp() != IsNow
    assert now.isoformat() == IsNow(iso_string=True)
    assert now.isoformat() != IsNow

    assert utc_now == IsNow(tz=timezone.utc)


def test_delta():
    assert IsNow(delta=timedelta(hours=2)).delta == timedelta(seconds=7200)
    assert IsNow(delta=3600).delta == timedelta(seconds=3600)
    assert IsNow(delta=3600.1).delta == timedelta(seconds=3600, microseconds=100000)


def test_is_now_relative(monkeypatch):
    mock = Mock(return_value=datetime(2020, 1, 1, 12, 13, 14))
    monkeypatch.setattr(IsNow, '_get_now', mock)
    assert IsNow() == datetime(2020, 1, 1, 12, 13, 14)


@pytest.mark.skipif(ZoneInfo is None, reason='requires zoneinfo')
def test_tz():
    new_year_london = datetime(2000, 1, 1, tzinfo=ZoneInfo('Europe/London'))

    new_year_eve_nyc = datetime(1999, 12, 31, 19, 0, 0, tzinfo=ZoneInfo('America/New_York'))

    assert new_year_eve_nyc == IsDatetime(approx=new_year_london, enforce_tz=False)
    assert new_year_eve_nyc != IsDatetime(approx=new_year_london, enforce_tz=True)

    new_year_naive = datetime(2000, 1, 1)

    assert new_year_naive != IsDatetime(approx=new_year_london, enforce_tz=False)
    assert new_year_naive != IsDatetime(approx=new_year_eve_nyc, enforce_tz=False)
    assert new_year_london == IsDatetime(approx=new_year_naive, enforce_tz=False)
    assert new_year_eve_nyc != IsDatetime(approx=new_year_naive, enforce_tz=False)


@pytest.mark.parametrize(
    'value,dirty,expect_match',
    [
        pytest.param(date(2000, 1, 1), IsDate(approx=date(2000, 1, 1)), True, id='same'),
        pytest.param('2000-01-01', IsDate(approx=date(2000, 1, 1), iso_string=True), True, id='iso-string-true'),
        pytest.param('2000-01-01', IsDate(approx=date(2000, 1, 1)), False, id='iso-string-different'),
        pytest.param('2000-01-01T00:00', IsDate(approx=date(2000, 1, 1)), False, id='iso-string-different'),
        pytest.param('broken', IsDate(approx=date(2000, 1, 1)), False, id='iso-string-wrong'),
        pytest.param('28/01/87', IsDate(approx=date(1987, 1, 28), format_string='%d/%m/%y'), True, id='string-format'),
        pytest.param('28/01/87', IsDate(approx=date(2000, 1, 1)), False, id='string-format-different'),
        pytest.param('foobar', IsDate(approx=date(2000, 1, 1)), False, id='string-format-wrong'),
        pytest.param([1, 2, 3], IsDate(approx=date(2000, 1, 1)), False, id='wrong-type'),
        pytest.param(
            datetime(2000, 1, 1, 10, 11, 12), IsDate(approx=date(2000, 1, 1)), False, id='wrong-type-datetime'
        ),
        pytest.param(date(2020, 1, 1), IsDate(approx=date(2020, 1, 1)), True, id='tz-same'),
        pytest.param(date(2000, 1, 1), IsDate(ge=date(2000, 1, 1)), True, id='ge'),
        pytest.param(date(1999, 1, 1), IsDate(ge=date(2000, 1, 1)), False, id='ge-not'),
        pytest.param(date(2000, 1, 2), IsDate(gt=date(2000, 1, 1)), True, id='gt'),
        pytest.param(date(2000, 1, 1), IsDate(gt=date(2000, 1, 1)), False, id='gt-not'),
        pytest.param(date(2000, 1, 1), IsDate(gt=date(2000, 1, 1), delta=10), False, id='delta-int'),
        pytest.param(date(2000, 1, 1), IsDate(gt=date(2000, 1, 1), delta=10.5), False, id='delta-float'),
        pytest.param(
            date(2000, 1, 1), IsDate(gt=date(2000, 1, 1), delta=timedelta(seconds=10)), False, id='delta-timedelta'
        ),
    ],
)
def test_is_date(value, dirty, expect_match):
    if expect_match:
        assert value == dirty
    else:
        assert value != dirty


def test_is_today():
    today = date.today()
    assert today == IsToday
    assert today + timedelta(days=2) != IsToday
    assert today.isoformat() == IsToday(iso_string=True)
    assert today.isoformat() != IsToday()
    assert today.strftime('%Y/%m/%d') == IsToday(format_string='%Y/%m/%d')
    assert today.strftime('%Y/%m/%d') != IsToday()



import pytest

from dirty_equals import AnyThing, Contains, HasLen, IsInt, IsList, IsListOrTuple, IsNegative, IsTuple


@pytest.mark.parametrize(
    'other,dirty',
    [
        ([], IsList),
        ((), IsTuple),
        ([], IsList()),
        ([1], IsList(length=1)),
        ((), IsTuple()),
        ([1, 2, 3], IsList(1, 2, 3)),
        ((1, 2, 3), IsTuple(1, 2, 3)),
        ((1, 2, 3), IsListOrTuple(1, 2, 3)),
        ([1, 2, 3], IsListOrTuple(1, 2, 3)),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=5)),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=(4, 6))),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=[4, 6])),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=(4, ...))),
        ([3, 2, 1], IsList(1, 2, 3, check_order=False)),
        ([{1: 2}, 7], IsList(7, {1: 2}, check_order=False)),
        ([1, 2, 3, 4], IsList(positions={0: 1, 2: 3, -1: 4})),
        ([1, 2, 3], IsList(AnyThing, 2, 3)),
        ([1, 2, 3], IsList(1, 2, IsInt)),
        ([3, 2, 1], IsList(1, 2, IsInt, check_order=False)),
        ([1, 2, 2], IsList(2, 2, 1, check_order=False)),
        ([], HasLen(0)),
        ([1, 2, 3], HasLen(3)),
        ('123', HasLen(3)),
        (b'123', HasLen(3)),
        ({'a': 1, 'b': 2, 'c': 3}, HasLen(3)),
        ([1, 2], HasLen(1, 2)),
        ([1, 2], HasLen(2, 3)),
        ([1, 2, 3], HasLen(2, ...)),
        ([1, 2, 3], HasLen(0, ...)),
        ([1, 2, 3], Contains(1)),
        ([1, 2, 3], Contains(1, 2)),
        ((1, 2, 3), Contains(1)),
        ({1, 2, 3}, Contains(1)),
        ('abc', Contains('b')),
        ({'a': 1, 'b': 2}, Contains('a')),
        ([{'a': 1}, {'b': 2}], Contains({'a': 1})),
    ],
)
def test_dirty_equals(other, dirty):
    assert other == dirty


@pytest.mark.parametrize(
    'other,dirty',
    [
        ([], IsTuple),
        ((), IsList),
        ([1], IsList),
        ([1, 2, 3], IsTuple(1, 2, 3)),
        ((1, 2, 3), IsList(1, 2, 3)),
        ([1, 2, 3, 4], IsList(1, 2, 3)),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=6)),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=(6, 8))),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=(0, 2))),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=[1, 2])),
        ([1, 2, 3, 4, 5], IsList(1, 2, 3, length=(6, ...))),
        ([3, 2, 1, 4], IsList(1, 2, 3, check_order=False)),
        ([1, 2, 3, 4], IsList(positions={0: 1, 2: 3, -1: 5})),
        ([1, 2, 3], IsList(1, 2, IsNegative)),
        ([1, 2, 2], IsList(1, 2, 3, check_order=False)),
        ([1, 2, 3], IsList(1, 2, 2, check_order=False)),
        ([1], HasLen(0)),
        ([], HasLen(1)),
        ('abc', HasLen(2)),
        ([1, 2, 3], HasLen(1, 2)),
        ([1], HasLen(2, 3)),
        ([1], HasLen(2, ...)),
        (123, HasLen(0, ...)),
        ([1, 2, 3], Contains(10)),
        ([1, 2, 3], Contains(1, 'a')),
        ([1, 2, 3], Contains(1, 'a')),
        ([{'a': 1}, {'b': 2}], Contains({'a': 2})),
        ({1, 2, 3}, Contains({1: 2})),
    ],
)
def test_dirty_not_equals(other, dirty):
    assert other != dirty


def test_args_and_positions():
    with pytest.raises(TypeError, match='IsList requires either args or positions, not both'):
        IsList(1, 2, positions={0: 1})


def test_positions_with_check_order():
    with pytest.raises(TypeError, match='check_order=False is not compatible with positions'):
        IsList(check_order=False, positions={0: 1})


def test_wrong_length_length():
    with pytest.raises(TypeError, match='length must be a tuple of length 2, not 3'):
        IsList(1, 2, length=(1, 2, 3))


@pytest.mark.parametrize(
    'dirty,repr_str',
    [
        (IsList, 'IsList'),
        (IsTuple(1, 2, 3), 'IsTuple(1, 2, 3)'),
        (IsList(positions={1: 10, 2: 20}), 'IsList(positions={1: 10, 2: 20})'),
        (IsTuple(1, 2, 3, length=4), 'IsTuple(1, 2, 3, length=4)'),
        (IsTuple(1, 2, 3, length=(6, ...)), 'IsTuple(1, 2, 3, length=(6, ...))'),
        (IsTuple(1, 2, 3, length=(6, 'x')), 'IsTuple(1, 2, 3, length=(6, ...))'),
        (IsTuple(1, 2, 3, length=(6, 10)), 'IsTuple(1, 2, 3, length=(6, 10))'),
        (IsTuple(1, 2, 3, check_order=False), 'IsTuple(1, 2, 3, check_order=False)'),
        (HasLen(42), 'HasLen(42)'),
        (HasLen(0, ...), 'HasLen(0, ...)'),
    ],
)
def test_repr(dirty, repr_str):
    assert repr(dirty) == repr_str


def test_no_contains_value():
    with pytest.raises(TypeError):
        Contains()



import pytest

from dirty_equals import IsDict, IsIgnoreDict, IsPartialDict, IsPositiveInt, IsStr, IsStrictDict


@pytest.mark.parametrize(
    'input_value,expected',
    [
        ({}, IsDict),
        ({}, IsDict()),
        ({'a': 1}, IsDict(a=1)),
        ({1: 2}, IsDict({1: 2})),
        ({'a': 1, 'b': 2}, IsDict(a=1, b=2)),
        ({'b': 2, 'a': 1}, IsDict(a=1, b=2)),
        ({'a': 1, 'b': None}, IsDict(a=1, b=None)),
        ({'a': 1, 'b': 3}, ~IsDict(a=1, b=2)),
        # partial dict
        ({1: 10, 2: 20}, IsPartialDict({1: 10})),
        ({1: 10}, IsPartialDict({1: 10})),
        ({1: 10, 2: 20}, IsPartialDict({1: 10})),
        ({1: 10, 2: 20}, IsDict({1: 10}).settings(partial=True)),
        ({1: 10}, ~IsPartialDict({1: 10, 2: 20})),
        ({1: 10, 2: None}, ~IsPartialDict({1: 10, 2: 20})),
        # ignore dict
        ({}, IsIgnoreDict()),
        ({'a': 1, 'b': 2}, IsIgnoreDict(a=1, b=2)),
        ({'a': 1, 'b': None}, IsIgnoreDict(a=1)),
        ({1: 10, 2: None}, IsIgnoreDict({1: 10})),
        ({'a': 1, 'b': 2}, ~IsIgnoreDict(a=1)),
        ({1: 10, 2: False}, ~IsIgnoreDict({1: 10})),
        ({1: 10, 2: False}, IsIgnoreDict({1: 10}).settings(ignore={False})),
        # strict dict
        ({}, IsStrictDict()),
        ({'a': 1, 'b': 2}, IsStrictDict(a=1, b=2)),
        ({'a': 1, 'b': 2}, ~IsStrictDict(b=2, a=1)),
        ({1: 10, 2: 20}, IsStrictDict({1: 10, 2: 20})),
        ({1: 10, 2: 20}, ~IsStrictDict({2: 20, 1: 10})),
        ({1: 10, 2: 20}, ~IsDict({2: 20, 1: 10}).settings(strict=True)),
        # combining types
        ({'a': 1, 'b': 2, 'c': 3}, IsStrictDict(a=1, c=3).settings(partial=True)),
        ({'a': 1, 'b': 2, 'c': 3}, IsStrictDict(a=1, b=2).settings(partial=True)),
        ({'a': 1, 'b': 2, 'c': 3}, IsStrictDict(b=2, c=3).settings(partial=True)),
        ({'a': 1, 'c': 3, 'b': 2}, ~IsStrictDict(b=2, c=3).settings(partial=True)),
    ],
)
def test_is_dict(input_value, expected):
    assert input_value == expected


def test_ne_repr_partial_dict():
    v = IsPartialDict({1: 10, 2: 20})

    with pytest.raises(AssertionError):
        assert 1 == v

    assert str(v) == 'IsPartialDict(1=10, 2=20)'


def test_ne_repr_strict_dict():
    v = IsStrictDict({1: 10, 2: 20})

    with pytest.raises(AssertionError):
        assert 1 == v

    assert str(v) == 'IsStrictDict(1=10, 2=20)'


def test_args_and_kwargs():
    with pytest.raises(TypeError, match='IsDict requires either a single argument or kwargs, not both'):
        IsDict(1, x=4)


def test_multiple_args():
    with pytest.raises(TypeError, match='IsDict expected at most 1 argument, got 2'):
        IsDict(1, 2)


def test_arg_not_dict():
    with pytest.raises(TypeError, match="expected_values must be a dict, got <class 'int'>"):
        IsDict(1)


def test_combine_partial_ignore():
    d = IsPartialDict(a=2, b=2, c=3)
    with pytest.raises(TypeError, match='partial and ignore cannot be used together'):
        d.settings(ignore={1})


def ignore_42(value):
    return value == 42


def test_callable_ignore():
    assert {'a': 1} == IsDict(a=1).settings(ignore=ignore_42)
    assert {'a': 1, 'b': 42} == IsDict(a=1).settings(ignore=ignore_42)
    assert {'a': 1, 'b': 43} != IsDict(a=1).settings(ignore=ignore_42)


@pytest.mark.parametrize(
    'd,expected_repr',
    [
        (IsDict, 'IsDict'),
        (IsDict(), 'IsDict()'),
        (IsPartialDict, 'IsPartialDict'),
        (IsPartialDict(), 'IsPartialDict()'),
        (IsDict().settings(partial=True), 'IsDict[partial=True]()'),
        (IsIgnoreDict(), 'IsIgnoreDict()'),
        (IsIgnoreDict().settings(ignore={7}), 'IsIgnoreDict[ignore={7}]()'),
        (IsIgnoreDict().settings(ignore={None}), 'IsIgnoreDict()'),
        (IsIgnoreDict().settings(ignore=None), 'IsIgnoreDict[ignore=None]()'),
        (IsDict().settings(ignore=ignore_42), 'IsDict[ignore=ignore_42]()'),
        (IsDict().settings(ignore={7}), 'IsDict[ignore={7}]()'),
        (IsDict().settings(ignore={None}), 'IsDict[ignore={None}]()'),
        (IsPartialDict().settings(partial=False), 'IsPartialDict[partial=False]()'),
        (IsStrictDict, 'IsStrictDict'),
        (IsStrictDict(), 'IsStrictDict()'),
        (IsDict().settings(strict=True), 'IsDict[strict=True]()'),
        (IsStrictDict().settings(strict=False), 'IsStrictDict[strict=False]()'),
    ],
)
def test_not_equals_repr(d, expected_repr):
    assert repr(d) == expected_repr


def test_ignore():
    def custom_ignore(v: int) -> bool:
        return v % 2 == 0

    assert {'a': 1, 'b': 2, 'c': 3, 'd': 4} == IsDict(a=1, c=3).settings(ignore=custom_ignore)


def test_ignore_with_is_str():
    api_data = {'id': 123, 'token': 't-abc123', 'dob': None, 'street_address': None}

    token_is_str = IsStr(regex=r't\-.+')
    assert api_data == IsIgnoreDict(id=IsPositiveInt, token=token_is_str)
    assert token_is_str.value == 't-abc123'


def test_unhashable_value():
    a = {'a': 1}
    api_data = {'b': a, 'c': None}
    assert api_data == IsIgnoreDict(b=a)



import platform
import pprint
from functools import singledispatch

import packaging.version
import pytest

from dirty_equals import Contains, DirtyEquals, IsApprox, IsInt, IsList, IsNegative, IsOneOf, IsPositive, IsStr
from dirty_equals.version import VERSION


def test_or():
    assert 'foo' == IsStr | IsInt
    assert 1 == IsStr | IsInt
    assert -1 == IsStr | IsNegative | IsPositive

    v = IsStr | IsInt
    with pytest.raises(AssertionError):
        assert 1.5 == v
    assert str(v) == 'IsStr | IsInt'


def test_and():
    assert 4 == IsPositive & IsInt(lt=5)

    v = IsStr & IsInt
    with pytest.raises(AssertionError):
        assert 1 == v
    assert str(v) == 'IsStr & IsInt'


def test_not():
    assert 'foo' != IsInt
    assert 'foo' == ~IsInt


def test_value_eq():
    v = IsStr()

    with pytest.raises(AttributeError, match='value is not available until __eq__ has been called'):
        v.value

    assert 'foo' == v
    assert repr(v) == str(v) == "'foo'" == pprint.pformat(v)
    assert v.value == 'foo'


def test_value_ne():
    v = IsStr()

    with pytest.raises(AssertionError):
        assert 1 == v

    assert repr(v) == str(v) == 'IsStr()' == pprint.pformat(v)
    with pytest.raises(AttributeError, match='value is not available until __eq__ has been called'):
        v.value


def test_dict_compare():
    v = {'foo': 1, 'bar': 2, 'spam': 3}
    assert v == {'foo': IsInt, 'bar': IsPositive, 'spam': ~IsStr}
    assert v == {'foo': IsInt() & IsApprox(1), 'bar': IsPositive() | IsNegative(), 'spam': ~IsStr()}


@pytest.mark.skipif(platform.python_implementation() == 'PyPy', reason='PyPy does not metaclass dunder methods')
def test_not_repr():
    v = ~IsInt
    assert str(v) == '~IsInt'

    with pytest.raises(AssertionError):
        assert 1 == v

    assert str(v) == '~IsInt'


def test_not_repr_instance():
    v = ~IsInt()
    assert str(v) == '~IsInt()'

    with pytest.raises(AssertionError):
        assert 1 == v

    assert str(v) == '~IsInt()'


def test_repr():
    v = ~IsInt
    assert str(v) == '~IsInt'

    assert '1' == v

    assert str(v) == "'1'"


@pytest.mark.parametrize(
    'v,v_repr',
    [
        (IsInt, 'IsInt'),
        (~IsInt, '~IsInt'),
        (IsInt & IsPositive, 'IsInt & IsPositive'),
        (IsInt | IsPositive, 'IsInt | IsPositive'),
        (IsInt(), 'IsInt()'),
        (~IsInt(), '~IsInt()'),
        (IsInt() & IsPositive(), 'IsInt() & IsPositive()'),
        (IsInt() | IsPositive(), 'IsInt() | IsPositive()'),
        (IsInt() & IsPositive, 'IsInt() & IsPositive'),
        (IsInt() | IsPositive, 'IsInt() | IsPositive'),
        (IsPositive & IsInt(lt=5), 'IsPositive & IsInt(lt=5)'),
        (IsOneOf(1, 2, 3), 'IsOneOf(1, 2, 3)'),
    ],
)
def test_repr_class(v, v_repr):
    assert repr(v) == str(v) == v_repr == pprint.pformat(v)


def test_is_approx_without_init():
    assert 1 != IsApprox


def test_ne_repr():
    v = IsInt
    assert repr(v) == str(v) == 'IsInt' == pprint.pformat(v)

    assert 'x' != v

    assert repr(v) == str(v) == 'IsInt' == pprint.pformat(v)


def test_pprint():
    v = [IsList(length=...), 1, [IsList(length=...), 2], 3, IsInt()]
    lorem = ['lorem', 'ipsum', 'dolor', 'sit', 'amet'] * 2
    with pytest.raises(AssertionError):
        assert [lorem, 1, [lorem, 2], 3, '4'] == v

    assert repr(v) == (f'[{lorem}, 1, [{lorem}, 2], 3, IsInt()]')
    assert pprint.pformat(v) == (
        "[['lorem',\n"
        "  'ipsum',\n"
        "  'dolor',\n"
        "  'sit',\n"
        "  'amet',\n"
        "  'lorem',\n"
        "  'ipsum',\n"
        "  'dolor',\n"
        "  'sit',\n"
        "  'amet'],\n"
        ' 1,\n'
        " [['lorem',\n"
        "   'ipsum',\n"
        "   'dolor',\n"
        "   'sit',\n"
        "   'amet',\n"
        "   'lorem',\n"
        "   'ipsum',\n"
        "   'dolor',\n"
        "   'sit',\n"
        "   'amet'],\n"
        '  2],\n'
        ' 3,\n'
        ' IsInt()]'
    )


def test_pprint_not_equal():
    v = IsList(*range(30))  # need a big value to trigger pprint
    with pytest.raises(AssertionError):
        assert [] == v

    assert (
        pprint.pformat(v)
        == (
            'IsList(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, '
            '15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29)'
        )
        == repr(v)
        == str(v)
    )


@pytest.mark.parametrize(
    'value,dirty',
    [
        (1, IsOneOf(1, 2, 3)),
        (4, ~IsOneOf(1, 2, 3)),
        ([1, 2, 3], Contains(1) | IsOneOf([])),
        ([], Contains(1) | IsOneOf([])),
        ([2], ~(Contains(1) | IsOneOf([]))),
    ],
)
def test_is_one_of(value, dirty):
    assert value == dirty


def test_version():
    packaging.version.parse(VERSION)


def test_singledispatch():
    @singledispatch
    def dispatch(value):
        return 'generic'

    assert dispatch(IsStr()) == 'generic'

    @dispatch.register
    def _(value: DirtyEquals):
        return 'DirtyEquals'

    assert dispatch(IsStr()) == 'DirtyEquals'

    @dispatch.register
    def _(value: IsStr):
        return 'IsStr'

    assert dispatch(IsStr()) == 'IsStr'



import re

import pytest

from dirty_equals import IsAnyStr, IsBytes, IsStr


@pytest.mark.parametrize(
    'value,dirty,match',
    [
        # IsStr tests
        ('foo', IsStr, True),
        ('foo', IsStr(), True),
        (b'foo', IsStr, False),
        ('foo', IsStr(regex='fo{2}'), True),
        ('foo', IsStr(regex=b'fo{2}'), False),
        ('foo', IsStr(regex=re.compile('fo{2}')), True),
        ('Foo', IsStr(regex=re.compile('fo{2}', flags=re.I)), True),
        ('Foo', IsStr(regex=re.compile('fo{2}'), regex_flags=re.I), True),
        ('foo', IsStr(regex='fo'), False),
        ('foo', IsStr(regex='foo', max_length=2), False),
        ('foo\nbar', IsStr(regex='fo.*', regex_flags=re.S), True),
        ('foo\nbar', IsStr(regex='fo.*'), False),
        ('foo', IsStr(min_length=3), True),
        ('fo', IsStr(min_length=3), False),
        ('foo', IsStr(max_length=3), True),
        ('foobar', IsStr(max_length=3), False),
        ('foo', IsStr(case='lower'), True),
        ('FOO', IsStr(case='lower'), False),
        ('FOO', IsStr(case='upper'), True),
        ('foo', IsStr(case='upper'), False),
        # IsBytes tests
        (b'foo', IsBytes, True),
        (b'foo', IsBytes(), True),
        ('foo', IsBytes, False),
        (b'foo', IsBytes(regex=b'fo{2}'), True),
        (b'Foo', IsBytes(regex=re.compile(b'fo{2}', flags=re.I)), True),
        (b'Foo', IsBytes(regex=re.compile(b'fo{2}'), regex_flags=re.I), True),
        (b'foo', IsBytes(regex=b'fo'), False),
        (b'foo', IsBytes(regex=b'foo', max_length=2), False),
        (b'foo\nbar', IsBytes(regex=b'fo.*', regex_flags=re.S), True),
        (b'foo\nbar', IsBytes(regex=b'fo.*'), False),
        (b'foo', IsBytes(min_length=3), True),
        (b'fo', IsBytes(min_length=3), False),
        # IsAnyStr tests
        (b'foo', IsAnyStr, True),
        (b'foo', IsAnyStr(), True),
        ('foo', IsAnyStr, True),
        (b'foo', IsAnyStr(regex=b'fo{2}'), True),
        ('foo', IsAnyStr(regex=b'fo{2}'), True),
        (b'foo', IsAnyStr(regex='fo{2}'), True),
        ('foo', IsAnyStr(regex='fo{2}'), True),
        (b'Foo', IsAnyStr(regex=re.compile(b'fo{2}', flags=re.I)), True),
        ('Foo', IsAnyStr(regex=re.compile(b'fo{2}', flags=re.I)), True),
        (b'Foo', IsAnyStr(regex=re.compile(b'fo{2}'), regex_flags=re.I), True),
        ('Foo', IsAnyStr(regex=re.compile(b'fo{2}'), regex_flags=re.I), True),
        (b'Foo', IsAnyStr(regex=re.compile('fo{2}', flags=re.I)), True),
        ('Foo', IsAnyStr(regex=re.compile('fo{2}', flags=re.I)), True),
        (b'Foo', IsAnyStr(regex=re.compile('fo{2}'), regex_flags=re.I), True),
        ('Foo', IsAnyStr(regex=re.compile('fo{2}'), regex_flags=re.I), True),
        (b'foo\nbar', IsAnyStr(regex=b'fo.*', regex_flags=re.S), True),
        (b'foo\nbar', IsAnyStr(regex=b'fo.*'), False),
        ('foo', IsAnyStr(regex=b'foo', max_length=2), False),
        (b'foo', IsAnyStr(regex=b'foo', max_length=2), False),
        (b'foo', IsAnyStr(min_length=3), True),
        ('foo', IsAnyStr(min_length=3), True),
        (b'fo', IsAnyStr(min_length=3), False),
    ],
)
def test_dirty_equals_true(value, dirty, match):
    if match:
        assert value == dirty
    else:
        assert value != dirty


def test_regex_true():
    assert 'whatever' == IsStr(regex='whatever')
    reg = IsStr(regex='wh.*er')
    assert 'whatever' == reg
    assert str(reg) == "'whatever'"


def test_regex_bytes_true():
    assert b'whatever' == IsBytes(regex=b'whatever')
    assert b'whatever' == IsBytes(regex=b'wh.*er')


def test_regex_false():
    reg = IsStr(regex='wh.*er')
    with pytest.raises(AssertionError):
        assert 'WHATEVER' == reg
    assert str(reg) == "IsStr(regex='wh.*er')"


def test_regex_false_type_error():
    assert 123 != IsStr(regex='wh.*er')

    reg = IsBytes(regex=b'wh.*er')
    with pytest.raises(AssertionError):
        assert 'whatever' == reg
    assert str(reg) == "IsBytes(regex=b'wh.*er')"


def test_is_any_str():
    assert 'foobar' == IsAnyStr
    assert b'foobar' == IsAnyStr
    assert 123 != IsAnyStr
    assert 'foo' == IsAnyStr(regex='foo')
    assert 'foo' == IsAnyStr(regex=b'foo')
    assert b'foo' == IsAnyStr(regex='foo')
    assert b'foo' == IsAnyStr(regex=b'foo')



import pytest

from dirty_equals import AnyThing, HasAttributes, HasName, HasRepr, IsInstance, IsInt, IsStr


class Foo:
    def __init__(self, a=1, b=2):
        self.a = a
        self.b = b

    def spam(self):
        pass


def dirty_repr(value):
    if hasattr(value, 'equals'):
        return repr(value)
    return ''


def test_is_instance_of():
    assert Foo() == IsInstance(Foo)
    assert Foo() == IsInstance[Foo]
    assert 1 != IsInstance[Foo]


class Bar(Foo):
    def __repr__(self):
        return f'Bar(a={self.a}, b={self.b})'


def test_is_instance_of_inherit():
    assert Bar() == IsInstance(Foo)
    assert Foo() == IsInstance(Foo, only_direct_instance=True)
    assert Bar() != IsInstance(Foo, only_direct_instance=True)

    assert Foo != IsInstance(Foo)
    assert Bar != IsInstance(Foo)
    assert type != IsInstance(Foo)


def test_is_instance_of_repr():
    assert repr(IsInstance) == 'IsInstance'
    assert repr(IsInstance(Foo)) == "IsInstance(<class 'tests.test_inspection.Foo'>)"


def even(x):
    return x % 2 == 0


@pytest.mark.parametrize(
    'value,dirty',
    [
        (Foo, HasName('Foo')),
        (Foo, HasName['Foo']),
        (Foo(), HasName('Foo')),
        (Foo(), ~HasName('Foo', allow_instances=False)),
        (Bar, ~HasName('Foo')),
        (int, HasName('int')),
        (42, HasName('int')),
        (even, HasName('even')),
        (Foo().spam, HasName('spam')),
        (Foo.spam, HasName('spam')),
        (Foo, HasName(IsStr(regex='F..'))),
        (Bar, ~HasName(IsStr(regex='F..'))),
    ],
    ids=dirty_repr,
)
def test_has_name(value, dirty):
    assert value == dirty


@pytest.mark.parametrize(
    'value,dirty',
    [
        (Foo(1, 2), HasAttributes(a=1, b=2)),
        (Foo(1, 's'), HasAttributes(a=IsInt(), b=IsStr())),
        (Foo(1, 2), ~HasAttributes(a=IsInt(), b=IsStr())),
        (Foo(1, 2), ~HasAttributes(a=1, b=2, c=3)),
        (Foo(1, 2), ~HasAttributes(a=1, b=2, missing=AnyThing)),
    ],
    ids=dirty_repr,
)
def test_has_attributes(value, dirty):
    assert value == dirty


@pytest.mark.parametrize(
    'value,dirty',
    [
        (Bar(1, 2), HasRepr('Bar(a=1, b=2)')),
        (Bar(1, 2), HasRepr['Bar(a=1, b=2)']),
        (4, ~HasRepr('Bar(a=1, b=2)')),
        (Foo(), HasRepr(IsStr(regex=r'<tests.test_inspection.Foo object at 0x[0-9a-f]{6,20}>'))),
        (Foo, HasRepr("<class 'tests.test_inspection.Foo'>")),
        (42, HasRepr('42')),
        (43, ~HasRepr('42')),
    ],
    ids=dirty_repr,
)
def test_has_repr(value, dirty):
    assert value == dirty



"""
This module is run with mypy to check types can be used correctly externally.
"""

import sys

sys.path.append('.')

from dirty_equals import HasName, HasRepr, IsStr

assert 123 == HasName('int')
assert 123 == HasRepr('123')
assert 123 == HasName(IsStr(regex='i..'))
assert 123 == HasRepr(IsStr(regex=r'\d{3}'))

# type ignore is required (if it wasn't, there would be an error)
assert 123 != HasName(123)  # type: ignore[arg-type]



import platform

import pytest

from dirty_equals import IsFalseLike, IsTrueLike


@pytest.mark.parametrize(
    'other, expected',
    [
        (False, IsFalseLike),
        (True, ~IsFalseLike),
        ([], IsFalseLike),
        ([1], ~IsFalseLike),
        ((), IsFalseLike),
        ('', IsFalseLike),
        ('', IsFalseLike(allow_strings=True)),
        ((1, 2), ~IsFalseLike),
        ({}, IsFalseLike),
        ({1: 'a'}, ~IsFalseLike),
        (set(), IsFalseLike),
        ({'a', 'b', 'c'}, ~IsFalseLike),
        (None, IsFalseLike),
        (0, IsFalseLike),
        (1, ~IsFalseLike),
        (0.0, IsFalseLike),
        (1.0, ~IsFalseLike),
        ('0', IsFalseLike(allow_strings=True)),
        ('1', ~IsFalseLike(allow_strings=True)),
        ('0.0', IsFalseLike(allow_strings=True)),
        ('0.000', IsFalseLike(allow_strings=True)),
        ('1.0', ~IsFalseLike(allow_strings=True)),
        ('False', IsFalseLike(allow_strings=True)),
        ('True', ~IsFalseLike(allow_strings=True)),
        (0, IsFalseLike(allow_strings=True)),
    ],
)
def test_is_false_like(other, expected):
    assert other == expected


def test_is_false_like_repr():
    assert repr(IsFalseLike) == 'IsFalseLike'
    assert repr(IsFalseLike()) == 'IsFalseLike()'
    assert repr(IsFalseLike(allow_strings=True)) == 'IsFalseLike(allow_strings=True)'


@pytest.mark.skipif(platform.python_implementation() == 'PyPy', reason='PyPy does not metaclass dunder methods')
def test_dirty_not_equals():
    with pytest.raises(AssertionError):
        assert 0 != IsFalseLike


def test_dirty_not_equals_instance():
    with pytest.raises(AssertionError):
        assert 0 != IsFalseLike()


def test_invalid_initialization():
    with pytest.raises(TypeError, match='takes 1 positional argument but 2 were given'):
        IsFalseLike(True)


@pytest.mark.parametrize(
    'other, expected',
    [
        (False, ~IsTrueLike),
        (True, IsTrueLike),
        ([], ~IsTrueLike),
        ([1], IsTrueLike),
        ((), ~IsTrueLike),
        ((1, 2), IsTrueLike),
        ({}, ~IsTrueLike),
        ({1: 'a'}, IsTrueLike),
        (set(), ~IsTrueLike),
        ({'a', 'b', 'c'}, IsTrueLike),
        (None, ~IsTrueLike),
        (0, ~IsTrueLike),
        (1, IsTrueLike),
        (0.0, ~IsTrueLike),
        (1.0, IsTrueLike),
    ],
)
def test_is_true_like(other, expected):
    assert other == expected



import platform
import sys
from pathlib import Path

import pytest
from pytest_examples import CodeExample, EvalExample, find_examples

root_dir = Path(__file__).parent.parent

examples = find_examples(
    root_dir / 'dirty_equals',
    root_dir / 'docs',
)


@pytest.mark.skipif(platform.python_implementation() == 'PyPy', reason='PyPy does not allow metaclass dunder methods')
@pytest.mark.skipif(sys.version_info >= (3, 12), reason="pytest-examples doesn't yet support 3.12")
@pytest.mark.parametrize('example', examples, ids=str)
def test_docstrings(example: CodeExample, eval_example: EvalExample):
    prefix_settings = example.prefix_settings()
    # E711 and E712 refer to `== True` and `== None` and need to be ignored
    # I001 refers is a problem with black and ruff disagreeing about blank lines :shrug:
    eval_example.set_config(ruff_ignore=['E711', 'E712', 'I001'])

    requires = prefix_settings.get('requires')
    if requires:
        requires_version = tuple(int(v) for v in requires.split('.'))
        if sys.version_info < requires_version:
            pytest.skip(f'requires python {requires}')

    if prefix_settings.get('test') != 'skip':
        if eval_example.update_examples:
            eval_example.run_print_update(example)
        else:
            eval_example.run_print_check(example)

    if prefix_settings.get('lint') != 'skip':
        if eval_example.update_examples:
            eval_example.format(example)
        else:
            eval_example.lint(example)
